%==========================================================
%  Input data
%==========================================================

disp('')
disp('=======================================')
disp('    Solving Poisson equation           ')
disp('=======================================')
disp('Domain size');
L=5                                 % Domain size [-L,L]x[-L,L]   
disp('Mesh resolutions');
Nres = [3,6,9,12,15,18,21,24,27,30] % Mesh resolutions
myimgsize ='-S720,480';             % Hardcopy image size
nrotframes = 72;                    % Number of frames for surface animation



%==========================================================
% Functions 
%==========================================================

% Exact solution
function z = uhat(x,y)
   z = x.^2 + x.*sin(y)+2;
end

% Source term in Poisson equation -∆u = f
function z=f(x,y)
   z = -2 +x.*sin(y);
end

% Dirichlet boundary conditions
function z = Diri(x,y)
    lx=length(x);
    ly=length(y);
    if lx > ly 
       y = ones(size(x))*y;
    elseif lx < ly
        x = ones(size(y))*x;
    end
    z = uhat(x,y);
end

% Helper function to generate file names
function name = framename(basename, i)
  name = sprintf('%s_%03d.png',basename,i);
end
  
%=====================================================================
% Solution and animation driver
%=====================================================================
%Size o subplots sr by sc
sr=1;
sc=3;
figure('Position',[100,100,400,300]);

Nx=[];  % Curent vector of mesh resolutionds for convergence graph
err = []; % Current vector of absolute error for given mesh resolution

i=0;  % hard vopy file ID number
for N = Nres    % loop over meshed
  fprintf(1,'\rSolving for (%d,%d) mesh',N+2,N+2);
  Uexact=zeros(N+2,N+2);
  hold on;

  % Solve equation
  [U,X,Y] = poissonfd(-L,L,-L,L,N,N,'f','Diri');

  %Calculate error of approximate versus exact solution
  [XX,YY] = meshgrid(X,Y);
  Uexact = uhat(XX,YY);
  err = [err, max(max(abs(Uexact - U)))];


  % Visualise the error
  Nx = [Nx,N];
  hp_e = subplot(sr,sc,1);
  get(hp_e,'position'); 
  set(hp_e,'position', [ 0.40580   0.24000   0.48420   0.836500]); 
  surface(XX,YY,Uexact-U);
  hcb=colorbar('northoutside');
  p =get(hcb,'position');
  p=p+[0,0.03,0,0];
  set(hcb,'position',p);
  xt=get(hcb,'xlim');
  set(hcb,'xtick',[xt(1),0,xt(2)]);
  axis([-L,L,-L,L,-1,1]); 
  zlh=zlabel('Error');
  box('off');
  set(zlh,'rotation',90);
  view(60, 12);

  % Draw error vs. mesh resolution graph
  hp_c=subplot(sr,sc,1);
  set(hp_c,'position', [0.11,0.12,0.85,0.85]) ;
  plot(Nx,err,'*-','linewidth',2);
  xlabel('Mesh density Nx=Ny');
  ylabel('Absolute error');
  axis([0,30,0,max(err)]);

  pause(1);

  % Save image frame of the convergence animation
  nm = framename('gh',i);
  i++;
  print(nm, myimgsize, '-tight');
  clf();
end

% Create animation of solution surface
disp('\nGenerating animated solution visualisation');
umin = min(min(U));
umax = max(max(U));
for azimuth=linspace(0,360,nrotframes)
hp_c=subplot(1,1,1);
set(hp_c,'position', [0.05,0.05,0.85,0.85]);
surface(X,Y,U);
title('Solution to Poisson problem -∆u=f(x,y) \nfor f(x,y)=-2+x*sin(y)' );
xlabel('x');
ylabel('y');
zlabel('u');
hcb=colorbar('southoutside');
p =get(hcb,'position');
p=p-[0,0.50,0,0];
  set(hcb,'position',[0,0,0,0]);
  axis([-L,L,-L,L,umin-0.01,umax+0.01],'square');
  view(azimuth,12);
  nm = framename('gh',i);
  i++;
  print(nm, myimgsize, '-tight');
end
disp('Done');
