function [u, x, y, error] = poissonfd(a, b, c, d, nx, ny, fun, bound, uex, varargin)
%
% POISSONFD two-dimensional Poisson solver
% [U, X, Y] = POISSONFD(A, B, C, D, NX, NY, FUN, BOUND) solves by five-point
% finite difference scheme the problem -LAPL(U) = FUN in the rectangle
% (A,B)x(C,D) with Dirichlet boundery conditions U(X,Y)=BOUND(X,Y) for any
% (X, Y) on the boundery of the rectangle.
%
% [U, X, Y, ERROR] = POISSONFD(A,B,C,D,NX,NY,FUN,BOUND,UEX) computes also the
% maximum nodal error ERROR with respect to the exact solution UEX. FUN, BOUND
% and UEX can be online functions.
%
% Parameters
% A, B
% C, D  rectangle (A,B)x(C,D) where the solution is computed.
% NX    X equispaced grid of NX intervals.
% NY    Y equispaced grid of NY intervals.
% FUN
% BOUND boundary condition.
% UEX   exact solution.
%
% Returns
% U     solution array.
% X     equispaced grid of NX intervals.
% Y     equispaced grid of NY intervals.
% ERROR maximum nodal error ERROR with respect to the exact solution UEX.
%

% This file is minor modification of poissonfd.m from NMLibforOctave
% avaiable at http://www.ipb.pt/~balsa/ 
% The modified and original version have swapped x and y axes in output U.
%
% Author: Carlos Balsa
% Modified by: Roman Putanowicz <putanowr AT L5.pk.edu.pl>
% Modification date: 2014 Mar 23 05:06:38 
global glob_a glob_b glob_c glob_d;
glob_a = a;
glob_b = b;
glob_c = c;
glob_d = d;

if nargin == 8
  uex = inline('0','x','y');
end
nx = nx+1;
ny = ny+1;
hx = (b-a)/nx;
hy = (d-c)/ny;
nx1 = nx+1;
ny1 = ny+1;
hx2 = hx^2;
hy2 = hy^2;
kii = 2/hx2+2/hy2;
kix = -1/hx2;
kiy = -1/hy2;
dim = (nx+1)*(ny+1);
K = speye(dim,dim);
rhs = zeros(dim,1);
y = c;
for m = 2:ny
  x = a;
  y = y+hy;
  for n = 2:nx
    i = n+(m-1)*(nx+1);
    x = x+hx;
    rhs(i) = feval(fun,x,y,varargin{:});
    K(i,i) = kii;
    K(i,i-1) = kix;
    K(i,i+1) = kix;
    K(i,i+nx1) = kiy;
    K(i,i-nx1) = kiy;
  end
end
rhs1 = zeros(dim,1);
x = linspace(a,b,nx1);
rhs1(1:nx1) = feval(bound,x,c,varargin{:});
rhs1(dim-nx:dim) = feval(bound,x,d, varargin{:});
y = linspace(c,d,ny1);
rhs1(1:nx1:dim-nx) = feval(bound,a,y,varargin{:});
rhs1(nx1:nx1:dim) = feval(bound, b, y, varargin{:});
rhs = rhs - K*rhs1;
nbound = [[1:nx1],[dim-nx:dim], [1:nx1:dim-nx],[nx1:nx1:dim]];
ninternal = setdiff([1:dim],nbound);
K = K(ninternal,ninternal);
rhs = rhs(ninternal);
utemp = K\rhs;
uh = rhs1;
uh(ninternal) = utemp;
k = 1;
y = c;
for j = 1:ny+1
  x = a;
  for i = 1:nx1
    u(j,i) = uh(k);
    k = k+1;
    ue(j,i) = feval(uex,x,y,varargin{:});
    x = x+hx;
  end
  y = y+hy;
end
x = [a:hx:b];
y = [c:hy:d];
if nargout == 4
  if nargin == 8
    warning('Exact solution not available');
    error = [];
  else
    error = max(max(abs(u-ue)))/max(max(abs(ue)));
  end
end
return;
